@@ -2,11 +2,11 @@ use std::{collections::HashMap, fmt::Debug, hash::Hash, time::Duration};
 
 pub(crate) use source::AveragingBuffer;
 use source::OneWayKalmanSourceController;
-use tracing::{debug, error, info};
+use tracing::{debug, error, info, warn};
 
 use crate::{
     clock::NtpClock,
-    config::{SourceDefaultsConfig, SynchronizationConfig},
+    config::{SourceConfig, SynchronizationConfig},
     packet::NtpLeapIndicator,
     system::TimeSnapshot,
     time_types::{NtpDuration, NtpTimestamp},
@@ -34,6 +34,11 @@ struct SourceSnapshot<Index: Copy> {
     state: KalmanState,
     wander: f64,
     delay: f64,
+    // The wraparound period of the source,
+    // that is, the smallest time duration where
+    // the source cant distinguish whether the offset
+    // is 0 or that time duration.
+    period: Option<f64>,
 
     source_uncertainty: NtpDuration,
     source_delay: NtpDuration,
@@ -84,7 +89,6 @@ pub struct KalmanClockController<C: NtpClock, SourceId: Hash + Eq + Copy + Debug
     sources: HashMap<SourceId, (Option<SourceSnapshot<SourceId>>, bool)>,
     clock: C,
     synchronization_config: SynchronizationConfig,
-    source_defaults_config: SourceDefaultsConfig,
     algo_config: AlgorithmConfig,
     freq_offset: f64,
     timedata: TimeSnapshot,
@@ -113,7 +117,10 @@ impl<C: NtpClock, SourceId: Hash + Eq + Copy + Debug> KalmanClockController<C, S
         }
         for (_, (state, _)) in self.sources.iter_mut() {
             if let Some(ref mut snapshot) = state {
-                snapshot.state = snapshot.state.progress_time(time, snapshot.wander)
+                snapshot.state =
+                    snapshot
+                        .state
+                        .progress_time(time, snapshot.wander, snapshot.period)
             }
         }
 
@@ -186,10 +193,22 @@ impl<C: NtpClock, SourceId: Hash + Eq + Copy + Debug> KalmanClockController<C, S
             };
 
             self.timedata.root_delay = combined.delay;
-            self.timedata.root_dispersion =
-                NtpDuration::from_seconds(combined.estimate.offset_variance().sqrt());
+            self.timedata.root_variance_base_time = time;
+            self.timedata.root_variance_base = combined.estimate.uncertainty.entry(0, 0);
+            self.timedata.root_variance_linear = combined.estimate.uncertainty.entry(0, 1);
+            self.timedata.root_variance_quadratic = combined.estimate.uncertainty.entry(1, 1);
+            self.timedata.root_variance_cubic = selection
+                .iter()
+                .map(|v| v.wander)
+                .fold(None, |v: Option<f64>, a: f64| {
+                    Some(v.map(|b| b.max(a)).unwrap_or(a))
+                })
+                .unwrap_or(self.algo_config.initial_wander);
             self.clock
-                .error_estimate_update(self.timedata.root_dispersion, self.timedata.root_delay)
+                .error_estimate_update(
+                    self.timedata.root_dispersion(time),
+                    self.timedata.root_delay,
+                )
                 .expect("Cannot update clock");
 
             if let Some(leap) = combined.leap_indicator {
@@ -262,10 +281,14 @@ impl<C: NtpClock, SourceId: Hash + Eq + Copy + Debug> KalmanClockController<C, S
                 .expect("Cannot adjust clock");
             for (state, _) in self.sources.values_mut() {
                 if let Some(ref mut state) = state {
-                    state.state = state.state.process_offset_steering(change);
+                    state.state = state.state.process_offset_steering(change, state.period);
                 }
             }
-            info!("Jumped offset by {}ms", change * 1e3);
+            if self.synchronization_config.warn_on_jump {
+                warn!("Jumped offset by {}ms. This may cause problems for other software. If this is not a problem for your system, you can reclassify this warning as an informative message through the `synchronization.warn_on_jump` setting in ntp.toml.", change * 1e3);
+            } else {
+                info!("Jumped offset by {}ms", change * 1e3);
+            }
             StateUpdate {
                 source_message: Some(KalmanControllerMessage {
                     inner: KalmanControllerMessageInner::Step { steer: change },
@@ -315,10 +338,12 @@ impl<C: NtpClock, SourceId: Hash + Eq + Copy + Debug> KalmanClockController<C, S
             .expect("Cannot adjust clock");
         for (state, _) in self.sources.values_mut() {
             if let Some(ref mut state) = state {
-                state.state =
-                    state
-                        .state
-                        .process_frequency_steering(freq_update, actual_change, state.wander)
+                state.state = state.state.process_frequency_steering(
+                    freq_update,
+                    actual_change,
+                    state.wander,
+                    state.period,
+                )
             }
         }
         debug!(
@@ -352,7 +377,6 @@ impl<C: NtpClock, SourceId: Hash + Eq + Copy + Debug + Send + 'static> TimeSyncC
     fn new(
         clock: C,
         synchronization_config: SynchronizationConfig,
-        source_defaults_config: SourceDefaultsConfig,
         algo_config: Self::AlgorithmConfig,
     ) -> Result<Self, C::Error> {
         // Setup clock
@@ -362,7 +386,6 @@ impl<C: NtpClock, SourceId: Hash + Eq + Copy + Debug + Send + 'static> TimeSyncC
             sources: HashMap::new(),
             clock,
             synchronization_config,
-            source_defaults_config,
             algo_config,
             freq_offset,
             desired_freq: 0.0,
@@ -377,12 +400,17 @@ impl<C: NtpClock, SourceId: Hash + Eq + Copy + Debug + Send + 'static> TimeSyncC
         Ok(())
     }
 
-    fn add_source(&mut self, id: SourceId) -> Self::NtpSourceController {
+    fn add_source(
+        &mut self,
+        id: SourceId,
+        source_config: SourceConfig,
+    ) -> Self::NtpSourceController {
         self.sources.insert(id, (None, false));
         KalmanSourceController::new(
             id,
             self.algo_config,
-            self.source_defaults_config,
+            None,
+            source_config,
             AveragingBuffer::default(),
         )
     }
@@ -390,13 +418,16 @@ impl<C: NtpClock, SourceId: Hash + Eq + Copy + Debug + Send + 'static> TimeSyncC
     fn add_one_way_source(
         &mut self,
         id: SourceId,
+        source_config: SourceConfig,
         measurement_noise_estimate: f64,
+        period: Option<f64>,
     ) -> Self::OneWaySourceController {
         self.sources.insert(id, (None, false));
         KalmanSourceController::new(
             id,
             self.algo_config,
-            self.source_defaults_config,
+            period,
+            source_config,
             measurement_noise_estimate,
         )
     }
@@ -495,14 +526,13 @@ mod tests {
             ..SynchronizationConfig::default()
         };
         let algo_config = AlgorithmConfig::default();
-        let source_defaults_config = SourceDefaultsConfig::default();
+        let source_config = SourceConfig::default();
         let mut algo = KalmanClockController::new(
             TestClock {
                 has_steered: RefCell::new(false),
                 current_time: NtpTimestamp::from_fixed_int(0),
             },
             synchronization_config,
-            source_defaults_config,
             algo_config,
         )
         .unwrap();
@@ -511,7 +541,7 @@ mod tests {
         // ignore startup steer of frequency.
         *algo.clock.has_steered.borrow_mut() = false;
 
-        let mut source = algo.add_source(0);
+        let mut source = algo.add_source(0, source_config);
         algo.source_update(0, true);
 
         assert!(algo.in_startup);
@@ -546,7 +576,7 @@ mod tests {
         assert!(!algo.in_startup);
         assert_eq!(algo.timedata.leap_indicator, NtpLeapIndicator::NoWarning);
         assert_ne!(algo.timedata.root_delay, NtpDuration::ZERO);
-        assert_ne!(algo.timedata.root_dispersion, NtpDuration::ZERO);
+        assert_ne!(algo.timedata.root_variance_base, 0.0);
     }
 
     #[test]
@@ -563,14 +593,12 @@ mod tests {
             step_threshold: 1800.0,
             ..Default::default()
         };
-        let source_defaults_config = SourceDefaultsConfig::default();
         let mut algo = KalmanClockController::<_, u32>::new(
             TestClock {
                 has_steered: RefCell::new(false),
                 current_time: NtpTimestamp::from_fixed_int(0),
             },
             synchronization_config,
-            source_defaults_config,
             algo_config,
         )
         .unwrap();
@@ -593,14 +621,12 @@ mod tests {
             ..SynchronizationConfig::default()
         };
         let algo_config = AlgorithmConfig::default();
-        let source_defaults_config = SourceDefaultsConfig::default();
         let mut algo = KalmanClockController::<_, u32>::new(
             TestClock {
                 has_steered: RefCell::new(false),
                 current_time: NtpTimestamp::from_fixed_int(0),
             },
             synchronization_config,
-            source_defaults_config,
             algo_config,
         )
         .unwrap();
@@ -614,14 +640,12 @@ mod tests {
     fn test_jumps_update_state() {
         let synchronization_config = SynchronizationConfig::default();
         let algo_config = AlgorithmConfig::default();
-        let source_defaults_config = SourceDefaultsConfig::default();
         let mut algo = KalmanClockController::<_, u32>::new(
             TestClock {
                 has_steered: RefCell::new(false),
                 current_time: NtpTimestamp::from_fixed_int(0),
             },
             synchronization_config,
-            source_defaults_config,
             algo_config,
         )
         .unwrap();
@@ -638,6 +662,29 @@ mod tests {
                     },
                     wander: 0.0,
                     delay: 0.0,
+                    period: None,
+                    source_uncertainty: NtpDuration::ZERO,
+                    source_delay: NtpDuration::ZERO,
+                    leap_indicator: NtpLeapIndicator::NoWarning,
+                    last_update: NtpTimestamp::from_fixed_int(0),
+                }),
+                true,
+            ),
+        );
+
+        algo.sources.insert(
+            1,
+            (
+                Some(SourceSnapshot {
+                    index: 0,
+                    state: KalmanState {
+                        state: Vector::new_vector([0.0, 0.0]),
+                        uncertainty: Matrix::new([[1e-18, 0.0], [0.0, 1e-18]]),
+                        time: NtpTimestamp::from_fixed_int(0),
+                    },
+                    wander: 0.0,
+                    delay: 0.0,
+                    period: Some(3.0),
                     source_uncertainty: NtpDuration::ZERO,
                     source_delay: NtpDuration::ZERO,
                     leap_indicator: NtpLeapIndicator::NoWarning,
@@ -652,6 +699,10 @@ mod tests {
             algo.sources.get(&0).unwrap().0.unwrap().state.offset(),
             -100.0
         );
+        assert_eq!(
+            algo.sources.get(&1).unwrap().0.unwrap().state.offset(),
+            -1.0
+        );
         assert_eq!(
             algo.sources.get(&0).unwrap().0.unwrap().state.time,
             NtpTimestamp::from_seconds_nanos_since_ntp_era(100, 0)
@@ -662,14 +713,12 @@ mod tests {
     fn test_freqsteer_update_state() {
         let synchronization_config = SynchronizationConfig::default();
         let algo_config = AlgorithmConfig::default();
-        let source_defaults_config = SourceDefaultsConfig::default();
         let mut algo = KalmanClockController::<_, u32>::new(
             TestClock {
                 has_steered: RefCell::new(false),
                 current_time: NtpTimestamp::from_fixed_int(0),
             },
             synchronization_config,
-            source_defaults_config,
             algo_config,
         )
         .unwrap();
@@ -686,6 +735,7 @@ mod tests {
                     },
                     wander: 0.0,
                     delay: 0.0,
+                    period: None,
                     source_uncertainty: NtpDuration::ZERO,
                     source_delay: NtpDuration::ZERO,
                     leap_indicator: NtpLeapIndicator::NoWarning,
@@ -707,14 +757,13 @@ mod tests {
             ..SynchronizationConfig::default()
         };
         let algo_config = AlgorithmConfig::default();
-        let source_defaults_config = SourceDefaultsConfig::default();
+        let source_config = SourceConfig::default();
         let mut algo = KalmanClockController::new(
             TestClock {
                 has_steered: RefCell::new(false),
                 current_time: NtpTimestamp::from_fixed_int(0),
             },
             synchronization_config,
-            source_defaults_config,
             algo_config,
         )
         .unwrap();
@@ -723,7 +772,7 @@ mod tests {
         // ignore startup steer of frequency.
         *algo.clock.has_steered.borrow_mut() = false;
 
-        let mut source = algo.add_source(0);
+        let mut source = algo.add_source(0, source_config);
         algo.source_update(0, true);
 
         let mut noise = 1e-9;
@@ -766,14 +815,13 @@ mod tests {
             ..SynchronizationConfig::default()
         };
         let algo_config = AlgorithmConfig::default();
-        let source_defaults_config = SourceDefaultsConfig::default();
+        let source_config = SourceConfig::default();
         let mut algo = KalmanClockController::new(
             TestClock {
                 has_steered: RefCell::new(false),
                 current_time: NtpTimestamp::from_fixed_int(0),
             },
             synchronization_config,
-            source_defaults_config,
             algo_config,
         )
         .unwrap();
@@ -782,7 +830,7 @@ mod tests {
         // ignore startup steer of frequency.
         *algo.clock.has_steered.borrow_mut() = false;
 
-        let mut source = algo.add_source(0);
+        let mut source = algo.add_source(0, source_config);
         algo.source_update(0, true);
 
         let mut noise = 1e-9;
