@@ -23,8 +23,6 @@ mod source;
 mod system;
 mod time_types;
 
-#[cfg(feature = "nts-pool")]
-mod nts_pool_ke;
 pub mod tls_utils;
 
 pub(crate) mod exitcode {
@@ -42,7 +40,7 @@ mod exports {
         TimeSyncController, TwoWayKalmanSourceController,
     };
     pub use super::clock::NtpClock;
-    pub use super::config::{SourceDefaultsConfig, StepThreshold, SynchronizationConfig};
+    pub use super::config::{SourceConfig, StepThreshold, SynchronizationConfig};
     pub use super::identifiers::ReferenceId;
     #[cfg(feature = "__internal-fuzz")]
     pub use super::ipfilter::fuzz::fuzz_ipfilter;
@@ -90,19 +88,9 @@ mod exports {
 
     pub use super::cookiestash::MAX_COOKIES;
 
-    #[cfg(feature = "ntpv5")]
     pub mod v5 {
         pub use crate::packet::v5::server_reference_id::{BloomFilter, ServerId};
     }
-
-    #[cfg(feature = "nts-pool")]
-    pub use super::nts_record::AeadAlgorithm;
-
-    #[cfg(feature = "nts-pool")]
-    pub use super::nts_pool_ke::{
-        ClientToPoolData, ClientToPoolDecoder, PoolToServerData, PoolToServerDecoder,
-        SupportedAlgorithmsDecoder,
-    };
 }
 
 #[cfg(feature = "__internal-api")]
