@@ -1,17 +1,19 @@
-#[cfg(feature = "ntpv5")]
-use crate::packet::{
-    v5::server_reference_id::{BloomFilter, RemoteBloomFilter},
-    ExtensionField, NtpHeader,
-};
 use crate::{
     algorithm::{ObservableSourceTimedata, SourceController},
-    config::SourceDefaultsConfig,
+    config::SourceConfig,
     cookiestash::CookieStash,
     identifiers::ReferenceId,
     packet::{Cipher, NtpAssociationMode, NtpLeapIndicator, NtpPacket, RequestIdentifier},
     system::{SystemSnapshot, SystemSourceUpdate},
     time_types::{NtpDuration, NtpInstant, NtpTimestamp, PollInterval},
 };
+use crate::{
+    packet::{
+        v5::server_reference_id::{BloomFilter, RemoteBloomFilter},
+        ExtensionField, NtpHeader,
+    },
+    NtpVersion,
+};
 use rand::{thread_rng, Rng};
 use serde::{Deserialize, Serialize};
 use std::{
@@ -25,7 +27,6 @@ use tracing::{debug, trace, warn};
 const MAX_STRATUM: u8 = 16;
 const POLL_WINDOW: std::time::Duration = std::time::Duration::from_secs(5);
 const STARTUP_TRIES_THRESHOLD: usize = 3;
-#[cfg(feature = "ntpv5")]
 const AFTER_UPGRADE_TRIES_THRESHOLD: u32 = 2;
 
 pub struct SourceNtsData {
@@ -71,6 +72,10 @@ pub struct NtpSource<Controller: SourceController<MeasurementDelay = NtpDuration
     // attacks and packet reordering.
     current_request_identifier: Option<(RequestIdentifier, NtpInstant)>,
 
+    // Whether we have seen a DENY/RSTR KISS response since the last succesfull
+    // interaction
+    have_deny_rstr_response: bool,
+
     stratum: u8,
     reference_id: ReferenceId,
 
@@ -81,13 +86,12 @@ pub struct NtpSource<Controller: SourceController<MeasurementDelay = NtpDuration
 
     controller: Controller,
 
-    source_defaults_config: SourceDefaultsConfig,
+    source_config: SourceConfig,
 
     buffer: [u8; 1024],
 
     protocol_version: ProtocolVersion,
 
-    #[cfg(feature = "ntpv5")]
     // TODO we only need this if we run as a server
     bloom_filter: RemoteBloomFilter,
 }
@@ -111,6 +115,23 @@ impl<Controller: SourceController<MeasurementDelay = ()>> OneWaySource<Controlle
     pub fn handle_message(&mut self, message: Controller::ControllerMessage) {
         self.controller.handle_message(message)
     }
+
+    pub fn observe<SourceId>(
+        &self,
+        name: String,
+        address: String,
+        id: SourceId,
+    ) -> ObservableSourceState<SourceId> {
+        ObservableSourceState {
+            timedata: self.controller.observe(),
+            unanswered_polls: 0,
+            poll_interval: crate::time_types::PollInterval::from_byte(0),
+            nts_cookies: None,
+            name,
+            address,
+            id,
+        }
+    }
 }
 
 #[derive(Debug, Copy, Clone)]
@@ -232,7 +253,6 @@ pub struct NtpSourceSnapshot {
 
     pub protocol_version: ProtocolVersion,
 
-    #[cfg(feature = "ntpv5")]
     pub bloom_filter: Option<BloomFilter>,
 }
 
@@ -241,7 +261,7 @@ impl NtpSourceSnapshot {
         &self,
         local_stratum: u8,
         local_ips: &[IpAddr],
-        #[cfg_attr(not(feature = "ntpv5"), allow(unused_variables))] system: &SystemSnapshot,
+        system: &SystemSnapshot,
     ) -> Result<(), AcceptSynchronizationError> {
         use AcceptSynchronizationError::*;
 
@@ -268,7 +288,6 @@ impl NtpSourceSnapshot {
             return Err(Loop);
         }
 
-        #[cfg(feature = "ntpv5")]
         match self.bloom_filter {
             Some(filter) if filter.contains_id(&system.server_id) => {
                 debug!("Source rejected because of detected synchronization loop (bloom filter)");
@@ -297,7 +316,6 @@ impl NtpSourceSnapshot {
             reach: source.reach,
             poll_interval: source.last_poll_interval,
             protocol_version: source.protocol_version,
-            #[cfg(feature = "ntpv5")]
             bloom_filter: source.bloom_filter.full_filter().copied(),
         }
     }
@@ -318,8 +336,7 @@ pub fn source_snapshot() -> NtpSourceSnapshot {
 
         reach,
         poll_interval: crate::time_types::PollIntervalLimits::default().min,
-        protocol_version: Default::default(),
-        #[cfg(feature = "ntpv5")]
+        protocol_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
         bloom_filter: None,
     }
 }
@@ -336,37 +353,30 @@ pub enum AcceptSynchronizationError {
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
 pub enum ProtocolVersion {
     V4,
-    #[cfg(feature = "ntpv5")]
-    V4UpgradingToV5 {
-        tries_left: u8,
-    },
-    #[cfg(feature = "ntpv5")]
+    V4UpgradingToV5 { tries_left: u8 },
     UpgradedToV5,
-    #[cfg(feature = "ntpv5")]
     V5,
 }
 
 impl ProtocolVersion {
-    pub fn is_expected_incoming_version(&self, incoming_version: u8) -> bool {
+    const DEFAULT_UPGRADE_TRIES: u8 = 8;
+
+    pub fn is_expected_incoming_version(&self, incoming_version: NtpVersion) -> bool {
         match self {
-            ProtocolVersion::V4 => incoming_version == 4 || incoming_version == 3,
-            #[cfg(feature = "ntpv5")]
-            ProtocolVersion::V4UpgradingToV5 { .. } => incoming_version == 4,
-            #[cfg(feature = "ntpv5")]
-            ProtocolVersion::UpgradedToV5 | ProtocolVersion::V5 => incoming_version == 5,
+            ProtocolVersion::V4 => {
+                incoming_version == NtpVersion::V4 || incoming_version == NtpVersion::V3
+            }
+            ProtocolVersion::V4UpgradingToV5 { .. } => incoming_version == NtpVersion::V4,
+            ProtocolVersion::UpgradedToV5 | ProtocolVersion::V5 => {
+                incoming_version == NtpVersion::V5
+            }
         }
     }
-}
 
-impl Default for ProtocolVersion {
-    #[cfg(feature = "ntpv5")]
-    fn default() -> Self {
-        Self::V4UpgradingToV5 { tries_left: 8 }
-    }
-
-    #[cfg(not(feature = "ntpv5"))]
-    fn default() -> Self {
-        Self::V4
+    pub fn v4_upgrading_to_v5_with_default_tries() -> ProtocolVersion {
+        ProtocolVersion::V4UpgradingToV5 {
+            tries_left: Self::DEFAULT_UPGRADE_TRIES,
+        }
     }
 }
 
@@ -470,7 +480,7 @@ pub struct ObservableSourceState<SourceId> {
 impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Controller> {
     pub(crate) fn new(
         source_addr: SocketAddr,
-        source_defaults_config: SourceDefaultsConfig,
+        source_config: SourceConfig,
         protocol_version: ProtocolVersion,
         controller: Controller,
         nts: Option<Box<SourceNtsData>>,
@@ -479,8 +489,10 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
             Self {
                 nts,
 
-                last_poll_interval: source_defaults_config.poll_interval_limits.min,
-                remote_min_poll_interval: source_defaults_config.poll_interval_limits.min,
+                last_poll_interval: source_config.poll_interval_limits.min,
+                remote_min_poll_interval: source_config.poll_interval_limits.min,
+
+                have_deny_rstr_response: false,
 
                 current_request_identifier: None,
                 source_id: ReferenceId::from_ip(source_addr.ip()),
@@ -491,14 +503,13 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
                 stratum: 16,
                 reference_id: ReferenceId::NONE,
 
-                source_defaults_config,
+                source_config,
                 controller,
 
                 buffer: [0; 1024],
 
                 protocol_version, // TODO make this configurable
 
-                #[cfg(feature = "ntpv5")]
                 bloom_filter: RemoteBloomFilter::new(16).expect("16 is a valid chunk size"),
             },
             actions!(NtpSourceAction::SetTimer(Duration::from_secs(0))),
@@ -523,13 +534,17 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
             .max(self.remote_min_poll_interval)
     }
 
-    #[cfg_attr(not(feature = "ntpv5"), allow(unused_mut))]
     pub fn handle_timer(&mut self) -> NtpSourceActionIterator<Controller::SourceMessage> {
         if !self.reach.is_reachable() && self.tries >= STARTUP_TRIES_THRESHOLD {
-            return actions!(NtpSourceAction::Reset);
+            if self.have_deny_rstr_response {
+                // There were kiss of death responses, so we should probably demobilize instead
+                // of just retrying endlessly
+                return actions!(NtpSourceAction::Demobilize);
+            } else {
+                return actions!(NtpSourceAction::Reset);
+            }
         }
 
-        #[cfg(feature = "ntpv5")]
         if matches!(self.protocol_version, ProtocolVersion::UpgradedToV5)
             && self.reach.unanswered_polls() >= AFTER_UPGRADE_TRIES_THRESHOLD
         {
@@ -550,15 +565,18 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
                 // when requesting new cookies. We keep 350
                 // bytes of margin for header, ids, extension
                 // field headers and signature.
-                let new_cookies = nts
-                    .cookies
-                    .gap()
-                    .min(((self.buffer.len() - 300) / cookie.len()).min(u8::MAX as usize) as u8);
+                let new_cookies = nts.cookies.gap().min(
+                    ((self.buffer.len() - 300) / (cookie.len().max(1))).min(u8::MAX as usize) as u8,
+                );
+                // Defence in depth, ensure we can get at least 1 new cookie.
+                if new_cookies == 0 {
+                    warn!("NTS Cookie too large, resetting source. This may be a problem with the source");
+                    return actions![NtpSourceAction::Reset];
+                }
                 match self.protocol_version {
                     ProtocolVersion::V4 => {
                         NtpPacket::nts_poll_message(&cookie, new_cookies, poll_interval)
                     }
-                    #[cfg(feature = "ntpv5")]
                     ProtocolVersion::V4UpgradingToV5 { .. }
                     | ProtocolVersion::V5
                     | ProtocolVersion::UpgradedToV5 => {
@@ -568,11 +586,9 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
             }
             None => match self.protocol_version {
                 ProtocolVersion::V4 => NtpPacket::poll_message(poll_interval),
-                #[cfg(feature = "ntpv5")]
                 ProtocolVersion::V4UpgradingToV5 { .. } => {
                     NtpPacket::poll_message_upgrade_request(poll_interval)
                 }
-                #[cfg(feature = "ntpv5")]
                 ProtocolVersion::UpgradedToV5 | ProtocolVersion::V5 => {
                     NtpPacket::poll_message_v5(poll_interval)
                 }
@@ -580,7 +596,6 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
         };
         self.current_request_identifier = Some((identifier, NtpInstant::now() + POLL_WINDOW));
 
-        #[cfg(feature = "ntpv5")]
         if let NtpHeader::V5(header) = packet.header() {
             let req_ef = self.bloom_filter.next_request(header.client_cookie);
             packet.push_additional(ExtensionField::ReferenceIdRequest(req_ef));
@@ -647,7 +662,7 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
             .is_expected_incoming_version(message.version())
         {
             warn!(
-                incoming_version = message.version(),
+                incoming_version = message.version().as_u8(),
                 expected_version = ?self.protocol_version,
                 "Received packet with unexpected version from source"
             );
@@ -664,7 +679,6 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
             }
         };
 
-        #[cfg(feature = "ntpv5")]
         if message.valid_server_response(request_identifier, self.nts.is_some()) {
             if let ProtocolVersion::V4UpgradingToV5 { tries_left } = self.protocol_version {
                 let tries_left = tries_left.saturating_sub(1);
@@ -696,15 +710,21 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
             // KISS packets may not have correct timestamps at all, handle them anyway
             self.remote_min_poll_interval = Ord::max(
                 self.remote_min_poll_interval
-                    .inc(self.source_defaults_config.poll_interval_limits),
+                    .inc(self.source_config.poll_interval_limits),
                 self.last_poll_interval,
             );
             warn!(?self.remote_min_poll_interval, "Source requested rate limit");
             actions!()
         } else if message.is_kiss_rstr() || message.is_kiss_deny() {
             warn!("Source denied service");
-            // KISS packets may not have correct timestamps at all, handle them anyway
-            actions!(NtpSourceAction::Demobilize)
+            // Handle the kiss if it was signed, otherwise ignore it
+            if self.nts.is_some() {
+                actions!(NtpSourceAction::Demobilize)
+            } else {
+                // Not signed, so easily faked, but do register it for future reference
+                self.have_deny_rstr_response = true;
+                actions!()
+            }
         } else if message.is_kiss_ntsn() {
             warn!("Received nts not-acknowledge");
             // as these can be easily faked, we dont immediately give up on receiving
@@ -741,6 +761,9 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
         // For reachability, mark that we have had a response
         self.reach.received_packet();
 
+        // Clear received deny/rstr kod
+        self.have_deny_rstr_response = false;
+
         // we received this packet, and don't want to accept future ones with this next_expected_origin
         self.current_request_identifier = None;
 
@@ -748,7 +771,6 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
         self.stratum = message.stratum();
         self.reference_id = message.reference_id();
 
-        #[cfg(feature = "ntpv5")]
         if let NtpHeader::V5(header) = message.header() {
             // Handle new requested poll interval
             let requested_poll = message.poll();
@@ -821,6 +843,8 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
 
             current_request_identifier: None,
 
+            have_deny_rstr_response: false,
+
             source_addr: SocketAddr::new(IpAddr::V4(Ipv4Addr::UNSPECIFIED), 0),
             source_id: ReferenceId::from_int(0),
             reach: Reach::default(),
@@ -829,14 +853,13 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
             stratum: 0,
             reference_id: ReferenceId::from_int(0),
 
-            source_defaults_config: SourceDefaultsConfig::default(),
+            source_config: SourceConfig::default(),
             controller,
 
             buffer: [0; 1024],
 
-            protocol_version: Default::default(),
+            protocol_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
 
-            #[cfg(feature = "ntpv5")]
             bloom_filter: RemoteBloomFilter::new(16).unwrap(),
         }
     }
@@ -844,13 +867,14 @@ impl<Controller: SourceController<MeasurementDelay = NtpDuration>> NtpSource<Con
 
 #[cfg(test)]
 mod test {
-    use crate::{packet::NoCipher, time_types::PollIntervalLimits, NtpClock};
+    use crate::{
+        packet::{AesSivCmac256, NoCipher},
+        time_types::PollIntervalLimits,
+        NtpClock,
+    };
 
     use super::*;
-    #[cfg(feature = "ntpv5")]
     use crate::packet::v5::server_reference_id::ServerId;
-    #[cfg(feature = "ntpv5")]
-    use rand::thread_rng;
 
     #[derive(Debug, Clone, Default)]
     struct TestClock {}
@@ -999,13 +1023,7 @@ mod test {
 
         let mut source = NtpSource::test_ntp_source(NoopController);
 
-        #[cfg_attr(not(feature = "ntpv5"), allow(unused_mut))]
-        let mut system = SystemSnapshot::default();
-
-        #[cfg(feature = "ntpv5")]
-        {
-            system.server_id = ServerId::new(&mut thread_rng());
-        }
+        let system = SystemSnapshot::default();
 
         macro_rules! accept {
             () => {{
@@ -1072,6 +1090,23 @@ mod test {
         assert!(source.current_poll_interval() >= source.controller.0);
     }
 
+    #[test]
+    fn test_oversize_cookie_doesnt_crash() {
+        let mut source = NtpSource::test_ntp_source(NoopController);
+        let mut ntsdata = SourceNtsData {
+            cookies: CookieStash::default(),
+            c2s: Box::new(AesSivCmac256::new([0; 32].into())),
+            s2c: Box::new(AesSivCmac256::new([0; 32].into())),
+        };
+        ntsdata.cookies.store(vec![0; 2048]);
+        ntsdata.cookies.store(vec![0; 2048]);
+        source.nts = Some(Box::new(ntsdata));
+        let actions = source.handle_timer();
+        for action in actions {
+            assert!(matches!(action, NtpSourceAction::Reset))
+        }
+    }
+
     #[test]
     fn test_handle_incoming() {
         let base = NtpInstant::now();
@@ -1305,6 +1340,7 @@ mod test {
             NtpTimestamp::from_fixed_int(0),
             NtpTimestamp::from_fixed_int(100),
         );
+        assert!(!source.have_deny_rstr_response);
         assert!(actions.next().is_none());
 
         let mut packet = NtpPacket::test();
@@ -1330,7 +1366,9 @@ mod test {
             NtpTimestamp::from_fixed_int(0),
             NtpTimestamp::from_fixed_int(100),
         );
-        assert!(matches!(actions.next(), Some(NtpSourceAction::Demobilize)));
+        assert!(source.have_deny_rstr_response);
+        source.have_deny_rstr_response = false;
+        assert!(actions.next().is_none());
 
         let mut packet = NtpPacket::test();
         packet.set_reference_id(ReferenceId::KISS_DENY);
@@ -1341,6 +1379,7 @@ mod test {
             NtpTimestamp::from_fixed_int(0),
             NtpTimestamp::from_fixed_int(100),
         );
+        assert!(!source.have_deny_rstr_response);
         assert!(actions.next().is_none());
 
         let mut packet = NtpPacket::test();
@@ -1366,7 +1405,9 @@ mod test {
             NtpTimestamp::from_fixed_int(0),
             NtpTimestamp::from_fixed_int(100),
         );
-        assert!(matches!(actions.next(), Some(NtpSourceAction::Demobilize)));
+        assert!(source.have_deny_rstr_response);
+        source.have_deny_rstr_response = false;
+        assert!(actions.next().is_none());
 
         let old_remote_interval = source.remote_min_poll_interval;
         let mut packet = NtpPacket::test();
@@ -1409,7 +1450,6 @@ mod test {
         assert!(source.remote_min_poll_interval >= old_remote_interval);
     }
 
-    #[cfg(feature = "ntpv5")]
     #[test]
     fn upgrade_state_machine_does_stop() {
         let mut source = NtpSource::test_ntp_source(NoopController);
@@ -1436,7 +1476,7 @@ mod test {
 
             let poll_len: usize = poll.len();
             let (poll, _) = NtpPacket::deserialize(&poll, &NoCipher).unwrap();
-            assert_eq!(poll.version(), 4);
+            assert_eq!(poll.version(), NtpVersion::V4);
             assert!(poll.is_upgrade());
 
             let response = NtpPacket::timestamp_response(
@@ -1479,11 +1519,10 @@ mod test {
         }
         let poll = outgoingbuf.unwrap();
         let (poll, _) = NtpPacket::deserialize(&poll, &NoCipher).unwrap();
-        assert_eq!(poll.version(), 4);
+        assert_eq!(poll.version(), NtpVersion::V4);
         assert!(!poll.is_upgrade());
     }
 
-    #[cfg(feature = "ntpv5")]
     #[test]
     fn upgrade_state_machine_does_upgrade() {
         let mut source = NtpSource::test_ntp_source(NoopController);
@@ -1509,7 +1548,7 @@ mod test {
 
         let poll_len = poll.len();
         let (poll, _) = NtpPacket::deserialize(&poll, &NoCipher).unwrap();
-        assert_eq!(poll.version(), 4);
+        assert_eq!(poll.version(), NtpVersion::V4);
         assert!(poll.is_upgrade());
 
         let response = NtpPacket::timestamp_response(
@@ -1554,7 +1593,7 @@ mod test {
         }
         let poll = outgoingbuf.unwrap();
         let (poll, _) = NtpPacket::deserialize(&poll, &NoCipher).unwrap();
-        assert_eq!(poll.version(), 5);
+        assert_eq!(poll.version(), NtpVersion::V5);
 
         let response = NtpPacket::timestamp_response(
             &SystemSnapshot::default(),
@@ -1583,7 +1622,6 @@ mod test {
         assert!(matches!(source.protocol_version, ProtocolVersion::V5));
     }
 
-    #[cfg(feature = "ntpv5")]
     #[test]
     fn upgrade_state_machine_does_fallback_after_upgrade() {
         let mut source = NtpSource::test_ntp_source(NoopController);
@@ -1609,7 +1647,7 @@ mod test {
 
         let poll_len = poll.len();
         let (poll, _) = NtpPacket::deserialize(&poll, &NoCipher).unwrap();
-        assert_eq!(poll.version(), 4);
+        assert_eq!(poll.version(), NtpVersion::V4);
         assert!(poll.is_upgrade());
 
         let response = NtpPacket::timestamp_response(
@@ -1655,7 +1693,7 @@ mod test {
             }
             let poll = outgoingbuf.unwrap();
             let (poll, _) = NtpPacket::deserialize(&poll, &NoCipher).unwrap();
-            assert_eq!(poll.version(), 5);
+            assert_eq!(poll.version(), NtpVersion::V5);
         }
 
         let actions = source.handle_timer();
@@ -1672,14 +1710,13 @@ mod test {
         let poll = outgoingbuf.unwrap();
         let (poll, _) = NtpPacket::deserialize(&poll, &NoCipher).unwrap();
         assert!(matches!(source.protocol_version, ProtocolVersion::V4));
-        assert_eq!(poll.version(), 4);
+        assert_eq!(poll.version(), NtpVersion::V4);
     }
 
-    #[cfg(feature = "ntpv5")]
     #[test]
     fn bloom_filters_will_synchronize_at_some_point() {
         let mut server_filter = BloomFilter::new();
-        server_filter.add_id(&ServerId::new(&mut thread_rng()));
+        server_filter.add_id(&ServerId::default());
 
         let mut client = NtpSource::test_ntp_source(NoopController);
         client.protocol_version = ProtocolVersion::V5;
